--------------------------------------------------------------------------
--  DLX PROCESSOR MODEL SUITE
--  Copyright (C) 1995, Martin Gumm
--  University of Stuttgart / Department of Computer Science / IPVR-ISE
--------------------------------------------------------------------------
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 1, or (at your option)
--  any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--------------------------------------------------------------------------
--  Last revision date : November 15 1995
--------------------------------------------------------------------------

--------------------------------------------------------------------------
--  *** SYNOPSYS synthesizable code (ver. 3.2.a) ***
--------------------------------------------------------------------------

--------------------------------------------------------------------------
--  Package for the data types and microcode used in the controller 
--  
--  (file control_types.vhd)
--------------------------------------------------------------------------

library IEEE;
USE IEEE.std_logic_1164.ALL;

package control_types_2 is

  type test_and_set_instr is (sgt, slt, sne, seq, sge, sle, none);
  --
  -- finite state names (64 different states)
  --
  type fsm_states is (reset_1, reset_2, fetch, re_fetch, branch_1, branch_2, 
                      branch_3, stop_fetch_dec_exec, i_adr_viol);
  --
  --
  type dec_2_states is (load_w_1, load_h_1, load_b_1,
 		        store_w_1, store_h_1, store_b_1); 
  type dec_1_states is (nop_1, memory, movi2s, movs2i, br_eqz, br_nez,
 		        jump, jump_lnk_1, jump_reg, jump_lnk_reg_1,
 		        trap_1, rfe_1,
                        add_1, sub_1, and_1, or_1, xor_1, lhi,
                        sll_1, srl_1, sra_1, seq_1,
 		        sne_1, slt_1, sle_1, sgt_1, sge_1,
 		        ill_opc,
 		        ill_rraf);

  type exc_type is (nop, iav);
  --
  -- state of entry_mux
  constant aluout : std_logic := '1';
  constant pc     : std_logic := '0';
  --
  --
  -- state of wb_mux
  constant c      : std_logic := '0';
  constant lmdr   : std_logic := '1';
  --
  --
  -- state of pc_mux
  constant adder : std_logic := '1';
  constant mar : std_logic := '0';
  --
  --
  constant mov : std_logic := '0';
  constant trap : std_logic := '1';
  --
  -- enable low-active
  constant enable : std_logic := '0';
  constant disable : std_logic := '1';
  --
  --
  -- pc_latch enable
  constant just_read            : std_logic_vector(0 to 1) := "11";
  constant read_from_mar        : std_logic_vector(0 to 1) := "01";
  constant read_from_adder      : std_logic_vector(0 to 1) := "10";
  --
  --
  --
  -- source for s1 bus (low activ signals)
  -- |a|icr|iar|tbr|X|mdr|immed|consta| (b/pc/mar not connected to s1 bus)
  -- X : icr or iar or tbr, for mov instructions
  --
  constant s1_a     : std_logic_vector(0 to 5) := "011111";
  constant s1_icr   : std_logic_vector(0 to 5) := "101111";
  constant s1_iar   : std_logic_vector(0 to 5) := "110111";
  constant s1_tbr   : std_logic_vector(0 to 5) := "111011";
  constant s1_immed : std_logic_vector(0 to 5) := "111101";
  constant s1_const : std_logic_vector(0 to 5) := "111110";
  constant s1_none  : std_logic_vector(0 to 5) := "111111"; -- don't care not allowed
                                                              -- (only one driver !)
  --
  -- source for s2 bus (low activ signals)
  -- |b|immed|Y|pc|mar|constb|
  -- (a/mdr/icr/tbr/iar not connected to s2 bus)
  -- Y : b or immed, depends on instruction
  -- adr_ctrl : switch address for s2 bus when store Rtype instr.
  --
  constant s2_b      : std_logic_vector(0 to 4) := "01111";  
  constant s2_immed  : std_logic_vector(0 to 4) := "10111";
  constant s2_Y      : std_logic_vector(0 to 4) := "11011";  
  constant s2_pc     : std_logic_vector(0 to 4) := "11101";
  constant s2_const  : std_logic_vector(0 to 4) := "11110";
  constant s2_none   : std_logic_vector(0 to 4) := "11111"; -- don't care not allowed
                                                             -- (only one driver !)
  --
  -- destination from dest bus (low activ signals)
  -- |aluout|icr|iar|tbr|X|pc|mar|mdr|
  -- X : icr or iar or tbr, for mov instruction
  --
  constant dest_aluout : std_logic_vector(0 to 5) := "011111";
  constant dest_icr   : std_logic_vector(0 to 5)  := "101111";
  constant dest_iar   : std_logic_vector(0 to 5)  := "110111";
  constant dest_tbr   : std_logic_vector(0 to 5)  := "111011";
  constant dest_mar   : std_logic_vector(0 to 5)  := "111101";
  constant dest_res   : std_logic_vector(0 to 5)  := "010001";
  constant dest_none  : std_logic_vector(0 to 5)  := "111111";
  constant dest_X     : std_logic_vector(0 to 5)  := "111110";
  --
  -- select constant 
  -- |sel_const(0:1)|
  --
  constant const_00    : std_logic_vector(0 to 1) := "00";
  constant const_01    : std_logic_vector(0 to 1) := "01";
  constant const_04    : std_logic_vector(0 to 1) := "10";
  constant const_10    : std_logic_vector(0 to 1) := "01";
  constant const_18    : std_logic_vector(0 to 1) := "10";
  constant const_dcare : std_logic_vector(0 to 1) := "--";
  --
  -- select immediate
  -- |size_imm|sign_imm|
  --
  constant imm_s16   : std_logic_vector(0 to 1) := "01";
  constant imm_u16   : std_logic_vector(0 to 1) := "00";
  constant imm_s26   : std_logic_vector(0 to 1) := "11";
  constant imm_u26   : std_logic_vector(0 to 1) := "10";
  constant imm_dcare : std_logic_vector(0 to 1) := "--";
  --
  --
  -- alu operation
  --
  constant alu_pass_s1   : std_logic_vector(0 to 3) := "0000";
  constant alu_pass_s2   : std_logic_vector(0 to 3) := "0001";
  constant alu_s1_add_s2 : std_logic_vector(0 to 3) := "0100";
  constant alu_s1_sub_s2 : std_logic_vector(0 to 3) := "0101";
  constant alu_sll_s1_s2 : std_logic_vector(0 to 3) := "1000";
  constant alu_srl_s1_s2 : std_logic_vector(0 to 3) := "1010";
  constant alu_sra_s1_s2 : std_logic_vector(0 to 3) := "1011";
  constant alu_s1_and_s2 : std_logic_vector(0 to 3) := "1100";
  constant alu_s1_or_s2  : std_logic_vector(0 to 3) := "1101";
  constant alu_s1_xor_s2 : std_logic_vector(0 to 3) := "1110";
  constant alu_dcare     : std_logic_vector(0 to 3) := "----";
  --
  -- exception bit enable or setting
  -- |set_s|ioc|irra|iav|dav|ovad|ovar|priv|
  --  
  constant exc_none  : std_logic_vector(0 to 7) := "11111111";
  constant exc_sbit  : std_logic_vector(0 to 7) := "01111111";
  constant exc_ioc   : std_logic_vector(0 to 7) := "10111111";
  constant exc_irra  : std_logic_vector(0 to 7) := "11011111";
  constant exc_iav   : std_logic_vector(0 to 7) := "11101111";
  constant exc_dav   : std_logic_vector(0 to 7) := "11110111";
  constant exc_ovad  : std_logic_vector(0 to 7) := "11111011";
  constant exc_ovar  : std_logic_vector(0 to 7) := "11111101";
  constant exc_priv  : std_logic_vector(0 to 7) := "11111110";
  --
  -- memory control signals (pad_out_en : low active)
  -- |dec_enab(0 to 1)|rw|pad_d_inout|pad_out_en|error|
  --
  -- dec_enab(0 to 1) : "11" = word access
  --                    "10" = half "
  --                    "01" = byte "
  --                    "00" = no access
  --
  constant mem_none  : std_logic_vector(0 to 5) := "001100";
  constant mem_ldst  : std_logic_vector(0 to 5) := "000100";
  constant mem_lw    : std_logic_vector(0 to 5) := "111100";
  constant mem_lh    : std_logic_vector(0 to 5) := "101100";
  constant mem_lb    : std_logic_vector(0 to 5) := "011100";
  constant mem_sw    : std_logic_vector(0 to 5) := "110000";
  constant mem_sh    : std_logic_vector(0 to 5) := "100000";
  constant mem_sb    : std_logic_vector(0 to 5) := "010000";
  constant mem_halt  : std_logic_vector(0 to 5) := "001110";
  constant mem_error : std_logic_vector(0 to 5) := "001101";  
  -- mdr shift control signals
  -- (not used in finite state machine, only ir_decoder_2 )
  --
  constant mdr_ctrl_none : std_logic_vector(0 to 2) := "0--";
  constant mdr_ctrl_lh   : std_logic_vector(0 to 2) := "100";
  constant mdr_ctrl_lb   : std_logic_vector(0 to 2) := "101";
  constant mdr_ctrl_sh   : std_logic_vector(0 to 2) := "110";
  constant mdr_ctrl_sb   : std_logic_vector(0 to 2) := "111";
 
  --
  
end control_types_2;

