--------------------------------------------------------------------------
--  DLX PROCESSOR MODEL SUITE
--  Copyright (C) 1995, Martin Gumm
--  University of Stuttgart / Department of Computer Science / IPVR-ISE
--------------------------------------------------------------------------
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 1, or (at your option)
--  any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--------------------------------------------------------------------------
--  Last revision date : November 15 1995
--------------------------------------------------------------------------

--------------------------------------------------------------------------
--  *** SYNOPSYS synthesizable code (ver. 3.2.a) ***
--------------------------------------------------------------------------

--------------------------------------------------------------------------
--  Entity declaration for controller
--
--  (file controller.vhd)
--------------------------------------------------------------------------

--------------------------------------------------------------------------
  --
  -- source for s1 bus (low activ signals)
  -- |a|icr|iar|tbr|mdr|immed|consta| (b/pc/mar not connected to s1 bus)
  -- X : icr or iar or tbr, for mov instructions
  --
  -- source for s2 bus (low activ signals)
  -- |b|immed|pc|mar|constb|  (a/mdr/icr/tbr/iar not connected to s2 bus)
  -- Y : b or immed, depends on instruction
  --
  -- destination from dest bus (low activ signals)
  -- |c|icr|iar|tbr|pc|mar|mdr|
  -- X : icr or iar or tbr, for mov instruction
  --
  -- select constant 
  -- |sel_const(0:1)|
  --
  -- select immediate
  -- |size_imm|sign_imm|
  --
  -- register file operation
  -- (ab<-rf, rf<-c)
  -- |ab_rf|rf_c|
  --
  -- alu operation(0 to 3)
  --
  -- exception bit set
  -- |s|ioc|irra|iav|dav|ovad|ovar|priv|
  --
  -- memory control signals (ir_en, pad_out_en : low active)
  -- |ir_en|addr_mux|mdr_mux|enab(0 to 3)|rw|pad_d_inout|pad_out_en|error|
  --
  -- mdr shift control signals mdr_sh_ctrl(0 to2)
  --     "0--" load/store word
  --     "100" load half 
  --     "101" load byte 
  --     "110" store half
  --     "111" store byte 
  -- 
--------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;

use WORK.dlx_types.all;
use WORK.dlx_instructions.all;
use WORK.control_types_2.all;


entity controller2 is 

  port (phi1, phi2  : in std_logic;                 -- two phase clock in
	reset : in std_logic;                       -- reset input
	halt  : in std_logic;                       -- halt input
	ready_data : in std_logic;                       -- mem. ready input
        ready_instr : in std_logic;
        --
	instr_in : in dlx_word;                     -- instruction input
	alu_zero : in std_logic;                    -- alu zero bit
	alu_neg  : in std_logic;                    -- alu negative bit
	alu_ovfl : in std_logic;                    -- alu overflow bit
	intrpt   : in std_logic;                    -- interrupt active
	super    : in std_logic;                    -- supervisor bit 
        --
	-- reg. address outputs
	--
        --
	-- control outputs
	--
	-- from fetch stage
	--
        ir_latch_en_out : out std_logic;
        fetch_mem_ctrl  : out std_logic;
        pc_mux_sel      : out std_logic;
	--

        a_b_latch_out : out std_logic;
        ir_1_latch : out std_logic;
        t_and_s    : out std_logic;
        test_result : out std_logic;
	rs1_out : out dlx_reg_addr;
	rs2_out : out dlx_reg_addr;	  
        reg_zero : in std_logic;
        reg_zero_sel : out std_logic_vector;
	--
	--
        alu_mux_1_sel : out std_logic_vector(0 to 1);   -- forwarded results
        alu_mux_2_sel : out std_logic_vector(0 to 1);   
        smdr_mux_sel  : out std_logic_vector(0 to 1);
	s1_enab     : out std_logic_vector(0 to 5);    -- select s1 source
	s2_enab     : out std_logic_vector(0 to 3);    -- select s2_source
	dest_enab   : out std_logic_vector(0 to 4);    -- select destination
	alu_op_sel  : out std_logic_vector(0 to 3);    -- alu operation
	const_sel   : out std_logic_vector(0 to 1);    -- select const for s1
	immed_sel   : out std_logic_vector(0 to 1);    -- select immediate fro
	exc_enab    : out std_logic_vector(0 to 8);    -- enable set exception
        icr_shifter : out std_logic;
        iar_mux     : out std_logic;
        iar_latch   : out std_logic;
        smdr_latch  : out std_logic;
        smdr_ctrl : out std_logic_vector(0 to 2);
        --
        adr_ls2_in  : in std_logic_vector(1 downto 0);
        mem_ctrl      : out std_logic_vector(0 to 7);
	entry_mux_sel : out std_logic;
        c_latch     : out std_logic;
        pc_latch    : out std_logic;
        lmdr_ctrl : out std_logic_vector(0 to 2);
        lmdr_latch  : out std_logic;
	--
        wb_mux_sel  : out std_logic;
        rf_in_out   : out std_logic;
	rd_out  : out dlx_reg_addr;
        --
        adr_msb_in : in std_logic);

end controller2;




