--------------------------------------------------------------------------
--  DLX PROCESSOR MODEL SUITE
--  Copyright (C) 1995, Martin Gumm
--  University of Stuttgart / Department of Computer Science / IPVR-ISE
--------------------------------------------------------------------------
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 1, or (at your option)
--  any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--------------------------------------------------------------------------
--  Last revision date : November 15 1995
--------------------------------------------------------------------------

--------------------------------------------------------------------------
--  *** SYNOPSYS synthesizable code (ver. 3.2.a) ***
--------------------------------------------------------------------------

--------------------------------------------------------------------------
--  Entity declaration for the datapath2
--
--  (file datapath2.vhd)
--------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;

use WORK.dlx_types.all;
use WORK.control_types_2.all;

entity datapath2 is
  port(signal data_in        : in  dlx_word;           -- data in from pads
       signal instr_in       : in dlx_word;            -- instruction in
       signal data_out       : out dlx_word;           -- data_out to pads
       signal data_addr_out  : out dlx_word;           -- data-addr. out to pads
       signal instr_addr_out : out dlx_word;           -- instr-addr.
       signal instr_out      : out dlx_word;           -- instr. reg. content
       signal alu_zero       : out std_logic;                     
       signal alu_negative   : out std_logic;
       signal alu_overflow   : out std_logic;
       signal icr_intrpt     : out std_logic;          -- at least one masked intrpt.
       signal icr_super    : out std_logic;          -- supervisor bit in icr
       signal icr_reset    : in std_logic;
       
       signal mar_adr_ls2  : out std_logic_vector(0 to 1); -- least 2 addr. bits
       signal mar_adr_msb  : out std_logic;          --  msb of mar
       --
       -- reg. addr. inputs
       --
       signal reg_addr_rs1 : in dlx_reg_addr;
       signal reg_addr_rs2 : in dlx_reg_addr;
       signal reg_addr_rd  : in dlx_reg_addr;
       --
       -- control inputs
       --
       signal a_latch_en  	: in std_logic;         -- low active
       signal a_out_en    	: in std_logic;         -- low active
       signal b_latch_en  	: in std_logic;         -- low active
       signal b_out_en    	: in std_logic;         -- low active
       signal ir_1_latch_en     : in std_logic;
       signal c_latch_en  	: in std_logic;         -- low active
       signal aluout_latch_en   : in std_logic;
       signal regf_wr_en  	: in std_logic;         -- low active
       signal reg_zero          : out std_logic;
       signal reg_zero_sel      : in std_logic_vector(0 to 1);

       signal mar_latch_en : in std_logic;        -- low active

       signal lmdr_latch_en : in std_logic;        -- low active
       signal smdr_latch_en : in std_logic;

       signal lmdr_ctrl  : in std_logic_vector(0 to 2); -- enab signal for shift op.

       signal smdr_ctrl  : in std_logic_vector(0 to 2);
       signal smdr_mux_sel : in std_logic_vector(0 to 1);

       signal pc_latch_en  : in std_logic;        -- low active
       signal pc_out_en    : in std_logic;        -- low active

       signal iar_latch_en : in std_logic;        -- low active
       signal iar_out_en   : in std_logic;        -- low active
       signal tbr_latch_en : in std_logic;        -- low active
       signal tbr_out_en   : in std_logic;        -- low active

       signal icr_out_en   : in std_logic;        -- low active
       signal icr_latch_en : in std_logic;        -- low active
       signal icr_s_en     : in std_logic;        -- enab set s bit (low act.)
       signal icr_ioc_en   : in std_logic;        -- enab set ioc bit (low act.)
       signal icr_irra_en  : in std_logic;        -- enab set irra bit(low act.)
       signal icr_iav_en   : in std_logic;        -- enab set iav bit (low act.)
       signal icr_dav_en   : in std_logic;        -- enab set dav bit (low act.)
       signal icr_ovad_en   : in std_logic;       -- enab set ovad bit(low act.)
       signal icr_ovar_en   : in std_logic;       -- enab set ovar bit(low act.)
       signal icr_priv_en   : in std_logic;       -- enab set priv bit (low act.)
       signal icr_intrpt_in : in dlx_nibble;      -- intrpt. input from pads
       signal icr_intrpt_en : in std_logic;       -- enable setting of intrpt bits 

       signal ir_latch_en    : in std_logic;      -- low active
       signal ir_immed_o1_en : in std_logic;      -- low active
       signal ir_immed_o2_en : in std_logic;      -- low active
       signal ir_immed_size  : in std_logic;      -- '0'-> 16 bit / '1'-> 26 bit
       signal ir_immed_sign  : in std_logic;      -- '0'-> unsign./ '1'-> sign.  

       signal alu_mux_1_sel : in std_logic_vector(0 to 1);
       signal alu_mux_2_sel : in std_logic_vector(0 to 1);
       signal alu_latch_en : in std_logic;        -- low active
       signal alu_func     : in dlx_nibble;

       signal const_o1_en : in std_logic;         -- enable constant for s1 (low act.)
       signal const_o2_en : in std_logic;         -- enable constant for s2 (low act.)
       signal const_sel   : in std_logic_vector(0 to 1);  -- select constant

       signal wb_mux_sel  : in std_logic;
       signal entry_mux_sel : in std_logic;
       signal iar_mux_sel   : in std_logic;
       signal t_and_s       : in std_logic;
       signal test_result   : in std_logic;
       signal pc_mux_sel    : in std_logic);              
end datapath2;



